import typing
import urllib.parse

import aiohttp

from .platform import Platform

_english_pronouns: dict[str, list[str]] = {
    "unspecified": [],
    "he": ["he", "him"],
    "she": ["she", "her"],
    "they": ["they", "them"],
    "any": ["any"],
    "other": ["other"],
    "ask": ["ask"],
    "avoid": ["use name"],
}


async def lookup(platform: Platform, identifiers: typing.Union[str, int, list[str], list[int]], pronouns=None) -> \
        typing.Union[dict[str, list[str]], dict[int, list[str]]]:
    """
    Sends a request to the PronounDB API to get the pronouns of one or multiple users.
    If more than 50 identifiers are passed, the wrapper will automatically split the request into multiple requests.

    :param platform: One of the supported platforms (see the Platform enum)
    :param identifiers: Account IDs on the platform
    :param pronouns: An optional parameter that allows specifying the desired language for the returned pronouns.
                     By default, English pronouns are used (see _english_pronouns).
                     You can use this argument to retrieve pronouns in other languages by providing
                     a dictionary with the corresponding pronouns for the desired language.
    :return: The pronouns of the users as a list of all the pronouns the users use
    """

    if pronouns is None:
        pronouns = _english_pronouns

    if not isinstance(identifiers, list):
        identifiers = [identifiers]

    def get_pronouns_set(pronoun: str) -> list[str]:
        if not pronoun in pronouns:
            raise ValueError(f'{pronoun} not in pronouns parameter.')

        return pronouns[pronoun]

    all_results = {}

    for i in range(0, len(identifiers), 50):
        batch: typing.Union[list[str], list[int]] = identifiers[i:i + 50]

        async with aiohttp.ClientSession() as session:
            async with session.get("https://pronoundb.org/api/v2/lookup?platform={0}&ids={1}".format(
                    platform.value,
                    urllib.parse.quote_plus(",".join(map(str, batch)))
            )) as resp:
                data = await resp.json()

                if "error" in data:
                    raise ValueError(f'{data["error"]}: {data["message"]}')

                for identifier in batch:
                    if len(data) == 0:
                        all_results[identifier] = get_pronouns_set("unspecified")
                        continue

                    user_data = data[str(identifier)]
                    english_set = user_data["sets"]["en"]

                    if len(english_set) == 1:
                        all_results[identifier] = get_pronouns_set(english_set[0])
                    else:
                        all_results[identifier] = [
                            get_pronouns_set(english_set[0])[0],
                            get_pronouns_set(english_set[1])[0]
                        ]

    return all_results



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pronoundb']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8,<4.0']

setup_kwargs = {
    'name': 'pronoundb',
    'version': '2.0.0',
    'description': 'API wrapper for the pronoundb.org API.',
    'long_description': '# PronounDB Python API\n\n![PyPI](https://img.shields.io/pypi/v/pronoundb?style=flat-square)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/pronoundb?style=flat-square)\n![PyPI - License](https://img.shields.io/pypi/l/pronoundb?style=flat-square)\n\nAPI wrapper for the pronoundb.org API.\n\n## Installation\n\n```bash\npip install pronoundb\n```\n\n## Examples\n\nlookup someone\'s pronouns by their discord id:\n\n```py\nfrom pronoundb import lookup, Platform\n\nlookup(Platform.DISCORD, 123456789012345678)\n# -> {123456789012345678: ["he", "him"]}\n```\n\nlookup someone\'s pronouns by their minecraft (java) uuid:\n\n```py\nfrom pronoundb import lookup, Platform\n\nlookup(Platform.MINECRAFT, "12345678-1234-1234-1234-123456789012")\n# -> {"12345678-1234-1234-1234-123456789012": ["they", "them"]}\n```\n\nlookup multiple users pronouns by their discord id:\n\n```py\nfrom pronoundb import lookup, Platform\n\nlookup(Platform.DISCORD, [123456789012345678, 987654321098765432])\n# -> {123456789012345678: ["he", "him"], 987654321098765432: ["she", "her"]}\n```\n\n## Supported Platforms\n\n- Discord\n- GitHub\n- Minecraft (Java)\n- Twitch\n- Twitter\n\n## Custom Pronouns (Version 2.0.0)\n\nBeginning with version 2.0.0 you can give the lookup function a list of pronouns to translate them for example.\n\n```py\nfrom pronoundb import lookup, Platform\n\nlookup(Platform.DISCORD, 123456789012345678, {\n    "unspecified": [],\n    "he": ["Er", "Ihn"],\n    "she": ["Sie", "Ihr"],\n    "they": ["They", "Them"],\n    "any": ["Jede"],\n    "other": ["Anderes"],\n    "ask": ["Frag"],\n    "avoid": ["Nutz Name"],\n})\n# -> {123456789012345678: ["Er", "Ihn"]}\n```\n\n## Contributing\n\nContributions to this library are always welcome and highly encouraged.\n\n## License\n\nThis project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details.\n',
    'author': 'SteffoSpieler',
    'author_email': 'steffo@steffospieler.de',
    'maintainer': 'SteffoSpieler',
    'maintainer_email': 'steffo@steffospieler.de',
    'url': 'https://gitlab.com/SteffoSpieler/pronoundb-library',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
